"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.htmlEntitiesToUnicode = void 0;
const utils_1 = require("../../utils/utils");
const mapping = {
    '&AElig;': '00C6',
    '&Aacute;': '00C1',
    '&Acirc;': '00C2',
    '&Agrave;': '00C0',
    '&Aring;': '00C5',
    '&Atilde;': '00C3',
    '&Auml;': '00C4',
    '&Ccedil;': '00C7',
    '&ETH;': '00D0',
    '&Eacute;': '00C9',
    '&Ecirc;': '00CA',
    '&Egrave;': '00C8',
    '&Euml;': '00CB',
    '&Iacute;': '00CD',
    '&Icirc;': '00CE',
    '&Igrave;': '00CC',
    '&Iuml;': '00CF',
    '&Ntilde;': '00D1',
    '&Oacute;': '00D3',
    '&Ocirc;': '00D4',
    '&Ograve;': '00D2',
    '&Oslash;': '00D8',
    '&Otilde;': '00D5',
    '&Ouml;': '00D6',
    '&THORN;': '00DE',
    '&Uacute;': '00DA',
    '&Ucirc;': '00DB',
    '&Ugrave;': '00D9',
    '&Uuml;': '00DC',
    '&Yacute;': '00DD',
    '&aacute;': '00E1',
    '&acirc;': '00E2',
    '&acute;': '00B4',
    '&aelig;': '00E6',
    '&agrave;': '00E0',
    '&aring;': '00E5',
    '&atilde;': '00E3',
    '&auml;': '00E4',
    '&brvbar;': '00A6',
    '&ccedil;': '00E7',
    '&cedil;': '00B8',
    '&cent;': '00A2',
    '&copy;': '00A9',
    '&curren;': '00A4',
    '&deg;': '00B0',
    '&divide;': '00F7',
    '&eacute;': '00E9',
    '&ecirc;': '00EA',
    '&egrave;': '00E8',
    '&eth;': '00F0',
    '&euml;': '00EB',
    '&frac12;': '00BD',
    '&frac14;': '00BC',
    '&frac34;': '00BE',
    '&iacute;': '00ED',
    '&icirc;': '00EE',
    '&iexcl;': '00A1',
    '&igrave;': '00EC',
    '&iquest;': '00BF',
    '&iuml;': '00EF',
    '&laquo;': '00AB',
    '&macr;': '00AF',
    '&micro;': '00B5',
    '&middot;': '00B7',
    '&nbsp;': '00A0',
    '&not;': '00AC',
    '&ntilde;': '00F1',
    '&oacute;': '00F3',
    '&ocirc;': '00F4',
    '&ograve;': '00F2',
    '&ordf;': '00AA',
    '&ordm;': '00BA',
    '&oslash;': '00F8',
    '&otilde;': '00F5',
    '&ouml;': '00F6',
    '&para;': '00B6',
    '&plusmn;': '00B1',
    '&pound;': '00A3',
    '&raquo;': '00BB',
    '&reg;': '00AE',
    '&sect;': '00A7',
    '&shy;': '00AD',
    '&sup1;': '00B9',
    '&sup2;': '00B2',
    '&sup3;': '00B3',
    '&szlig;': '00DF',
    '&thorn;': '00FE',
    '&times;': '00D7',
    '&uacute;': '00FA',
    '&ucirc;': '00FB',
    '&ugrave;': '00F9',
    '&uml;': '00A8',
    '&uuml;': '00FC',
    '&yacute;': '00FD',
    '&yen;': '00A5',
    '&yuml;': '00FF',
};
function htmlEntitiesToUnicode(input) {
    return utils_1.findReplace(input, /(&[a-z0-9]+;)/gi, match => {
        const key = match[0];
        if (mapping[key])
            return '\\u' + mapping[key];
        return key;
    });
}
exports.htmlEntitiesToUnicode = htmlEntitiesToUnicode;
